﻿Imports System.ComponentModel
Imports System.Reactive.Concurrency
Imports System.Reactive.PlatformServices
Imports System.Reactive.Linq
Imports System.Reactive.Subjects
Imports System.Text
Imports System.Threading.Tasks
Imports System.Windows.Threading
Imports System
Imports System.Collections.Generic
Imports System.Diagnostics
Imports System.Linq
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Data
Imports System.Windows.Documents
Imports System.Windows.Input
Imports System.Windows.Media
Imports System.Windows.Media.Imaging
Imports System.Windows.Shapes

Imports Microsoft.Research.DynamicDataDisplay
Imports Microsoft.Research.DynamicDataDisplay.DataSources

Imports ODataSampleClient.Utilities

Namespace ODataSampleClient
	''' <summary>
	''' Interaction logic for DataDisplay.xaml
	''' </summary>
	Partial Public Class DataDisplay
		Inherits Window
#Region "Private fields"
		''' <summary>
		''' We can form connections to the OData service dynamically.
		''' </summary>
		Private dataLocation As Uri

		''' <summary>
		''' The subscription in the OData service we are monitoring.
		''' </summary>
		Private subscriptionNo As Integer

		''' <summary>
		''' When starting, collect the items currently monitored in this subscription.
		''' </summary>
		Private itemsInSubscription As List(Of Tuple(Of String, Integer))

		''' <summary>
		''' For each item monitored in this subscription, create a graphing sample buffer.
		''' </summary>
		Private itemSamplesFromSubscription As List(Of ItemSampleData)

		''' <summary>
		''' The shared x-axis data buffer.
		''' </summary>
		Private ReadOnly animatedX(999) As Double

		''' <summary>Programmatically created header</summary>
		Private chartHeader As New Header()

		''' <summary>Text contents of header</summary>
		Private headerContents As New TextBlock()

		''' <summary>Timer to animate data</summary>
		Private ReadOnly timer As New DispatcherTimer()
		Private scheduler As Reactive.Concurrency.SynchronizationContextScheduler = Nothing
#End Region

#Region "Internal class ItemSampleData"
		''' <summary>
		''' For a single item, capture the current sliding window
		''' of samples in a format acceptable to the DynamicDataDisplay library.
		''' </summary>
		Public Class ItemSampleData
			Implements IObserver(Of List(Of TimeSeriesDataFeed.Sample))
#Region "Private fields"
			''' <summary>
			''' The item id of the item being monitored.
			''' </summary>
			'INSTANT VB NOTE: The variable itemId was renamed since Visual Basic does not allow class members with the same name:
			Private itemId_Renamed As Integer

			''' <summary>
			''' The buffer we have linked to DynamicDataDisplay for a pen.
			''' </summary>
			Private ReadOnly animatedY(999) As Double

			''' <summary>
			''' A working buffer to accumulate samples from the service into.
			''' </summary>
			'INSTANT VB NOTE: The variable samples was renamed since Visual Basic does not allow class members with the same name:
			Private samples_Renamed As New List(Of Double)()

			''' <summary>
			''' Each channel offsets the raw data by a fixed amount.
			''' </summary>
			Private offset As Double = 0

			''' <summary>
			''' The DynamicDataDisplay data source for this pen.
			''' </summary>
			'INSTANT VB NOTE: The variable itemSamples was renamed since Visual Basic does not allow class members with the same name:
			Private itemSamples_Renamed As EnumerableDataSource(Of Double) = Nothing

			Private sampleFeed As IObservable(Of List(Of TimeSeriesDataFeed.Sample)) = Nothing
			Private scheduler As Reactive.Concurrency.SynchronizationContextScheduler = Nothing
#End Region

#Region "Public properties"
			''' <summary>
			''' The item id that this pen channel follows.
			''' </summary>
			Public ReadOnly Property ItemId() As Integer
				Get
					Return itemId_Renamed
				End Get
			End Property

			''' <summary>
			''' Gets a reference to the DynamicDataDisplay data source.
			''' </summary>
			Public ReadOnly Property ItemSamples() As EnumerableDataSource(Of Double)
				Get
					Return itemSamples_Renamed
				End Get
			End Property

			''' <summary>
			''' Gets a reference to the raw samples buffer.
			''' </summary>
			Public ReadOnly Property Samples() As List(Of Double)
				Get
					Return samples_Renamed
				End Get
			End Property
#End Region

#Region "Constructor"
			''' <summary>
			''' Construct the raw buffer and DynamicDataDisplay buffer
			''' for the pen channel for the item whose Id is given.
			''' </summary>
			''' <param name="id">The OData item id associated with this pen channel</param>
			'INSTANT VB TODO TASK: The following line could not be converted:
			Public Sub ItemSampleData(ByRef subscription As ODataSubscription, ByRef s As SynchronizationContextScheduler, ByVal id As Integer, ByRef offset As Double)
				' Capture the item id and offset for the pen channel.
				itemId_Renamed = id
				Me.offset = offset
				scheduler = s

				' Initialize the internal raw and DyanamicDataDisplay buffers to zeros.
				For i As Integer = 0 To animatedY.Length - 1
					animatedY(i) = 0
					samples_Renamed.Add(0)
				Next i

				' Make a DynamicDataDisplay data source and link it to the buffer.
				itemSamples_Renamed = New EnumerableDataSource(Of Double)(animatedY)
				itemSamples_Renamed.SetYMapping(Function(y) y)

				Try
					sampleFeed = subscription.GetItemSampleFeed(id)
					sampleFeed.Subscribe(Me)
				Catch e1 As ArgumentOutOfRangeException
					sampleFeed = Nothing
				End Try
			End Sub
#End Region

#Region "IObserver<TimeSeriesDataFeed.Sample> interface implementation"
			' Summary:
			'     Notifies the observer that the provider has finished sending push-based notifications.
			Public Sub OnCompleted() Implements IObserver(Of List(Of TimeSeriesDataFeed.Sample)).OnCompleted
			End Sub

			'
			' Summary:
			'     Notifies the observer that the provider has experienced an error condition.
			'
			' Parameters:
			'   error:
			'     An object that provides additional information about the error.
			Public Sub OnError([error] As Exception) Implements IObserver(Of List(Of TimeSeriesDataFeed.Sample)).OnError
			End Sub

			'
			' Summary:
			'     Provides the observer with new data.
			'
			' Parameters:
			'   value:
			'     The current notification information.
			Public Sub OnNext(values As List(Of TimeSeriesDataFeed.Sample)) Implements IObserver(Of List(Of TimeSeriesDataFeed.Sample)).OnNext
				samples_Renamed.AddRange((
					From value In values
					Select CDbl(value.SampleValue) + offset).ToList())
			End Sub
#End Region

#Region "Private helper methods"
			''' <summary>
			''' Dress the raw samples buffer, copy it to the DynamicDataDisplay buffer,
			''' and rais the data changed event.
			''' </summary>
			Public Sub RaiseDataChange()
				' If the raw buffer goes over the length of the DynamicDataDisplay buffer, trim it from the back.
				If samples_Renamed.Count > animatedY.Length Then
					samples_Renamed.RemoveRange(0, samples_Renamed.Count - animatedY.Length)
				End If

				' If we have the correct number of samples, copy the raw buffer to the DynamicDataDisplay buffer.
				If samples_Renamed.Count = animatedY.Length Then
					samples_Renamed.CopyTo(animatedY, 0)

					' Here it is - signal that data is updated
					itemSamples_Renamed.RaiseDataChanged()
				End If
			End Sub
#End Region

		End Class
#End Region

#Region "Constructor"
		''' <summary>
		''' Initializes a new instance of the <see cref="DataDisplay"/> class.
		''' This represents a single OData subscription display, connecting
		''' all items to their own display channels on a DynamicDataDisplay pen chart.
		''' </summary>
		''' <param name="dataSourceLocation">The base URL to the ODATA source</param>
		''' <param name="subs">The subscription Id of the subscription to monitor</param>
		Public Sub DataDisplay(ByRef dataSourceLocation As Uri, ByVal subs As Integer)
			' Capture the initializing parameters.
			dataLocation = dataSourceLocation
			subscriptionNo = subs
			itemsInSubscription = New List(Of Tuple(Of String, Integer))()
			itemSamplesFromSubscription = New List(Of ItemSampleData)()

			' Let the base class initialize this Window.
			InitializeComponent()

			scheduler = New SynchronizationContextScheduler(System.Threading.SynchronizationContext.Current)

			' Set up the pen chart.
			headerContents.FontSize = 24
			headerContents.Text = "Subscription " & subscriptionNo
			headerContents.HorizontalAlignment = HorizontalAlignment.Center
			chartHeader.Content = headerContents
			plotter.Children.Add(chartHeader)

			' Set a handler into the Loaded event, continue setup after the Window is initialized.

            AddHandler MyBase.Loaded, AddressOf Window_Loaded

			' Handle closing notification to clean up.
			AddHandler MyBase.Closing, AddressOf Window_Closing
		End Sub
#End Region

#Region "Plotting timer handler"
		''' <summary>
		''' Periodically poll all items in the subscription, and
		''' update their pen chart displays.
		''' </summary>
		''' <param name="sender"></param>
		''' <param name="e"></param>
        Private Sub AnimatedPlot_Timer(sender As Object, e As EventArgs)
            ' Sanity check that we have subscription information.
            If itemsInSubscription Is Nothing OrElse itemsInSubscription.Count = 0 Then
                Return
            End If

            ' Notify each plotter to redraw.
            For Each item In itemSamplesFromSubscription
                item.RaiseDataChange()
            Next item
        End Sub
#End Region

#Region "Window Loaded and Window Closing event handlers"
		''' <summary>
		''' Loaded event handler.  When the WPF form is loaded,
		''' this event handler will discover the items in the
		''' subscription, and initialize the pen channels for each item.
		''' </summary>
		''' <param name="sender"></param>
		''' <param name="e"></param>
        Private Sub Window_Loaded(sender As Object, e As RoutedEventArgs)
            ' Discover the items in this subscription.
            GetItems()

            ' Set up the X axis labels.
            For i As Integer = 0 To animatedX.Length - 1
                animatedX(i) = i
            Next i

            ' Make a DynamicDataDisplay shared source for the X axis.
            Dim xSrc As New EnumerableDataSource(Of Double)(animatedX)
            xSrc.SetXMapping(Function(x) x)

            ' Get a reference to the subscription feed for this subscription id.
            ODataSubscription.AddSubscription(subscriptionNo)
            Dim subscription As ODataSubscription = ODataSubscription.GetSubscription(subscriptionNo)

            ' Set up an ItemSampleData object to handle samples and pen channel for each item.
            If itemsInSubscription IsNot Nothing AndAlso itemsInSubscription.Count > 0 Then
                Dim offset As Double = 0
                For Each itemInSubscription In itemsInSubscription
                    Dim itemData As New ItemSampleData()
                    itemData.ItemSampleData(subscription, scheduler, itemInSubscription.Item2, offset)
                    itemSamplesFromSubscription.Add(itemData)

                    ' Adding graph to plotter
                    plotter.AddLineGraph(New CompositeDataSource(xSrc, itemData.ItemSamples), New Pen(ColorPicker.NextColor, 2), New PenDescription(itemInSubscription.Item1 & "(" & itemInSubscription.Item2 & ")"))

                    ' Each channel is separated by this much.
                    offset += 250
                Next itemInSubscription
            End If

            ' Turn on the redraw timer.
            timer.Interval = TimeSpan.FromMilliseconds(1000)
            AddHandler timer.Tick, AddressOf AnimatedPlot_Timer
            timer.IsEnabled = True

            ' Force evertyhing plotted to be visible
            plotter.FitToView()
        End Sub

		''' <summary>
		''' Query the OData source, retrieving a list of all items in the
		''' subscription that we monitor.
		''' </summary>
		Private Sub GetItems()
			' Always initialize a new context unless you are re-Imports the query.
			Dim dataContext As New TimeSeriesDataFeed.TimeSeriesEntities(dataLocation)

			' Generate the URL http://localhost:10001/TimeSeriesData.svc/Subscriptions(subscriptionNo)?$expand=Items
			Dim subscriptionWithItems = From subscription In dataContext.Subscriptions.Expand("Items")
										Where subscription.SubscriptionId = subscriptionNo
										Select subscription

			' Execute the query.  There should be exactly one subscription, select only the /Items collection.
			Dim itemsInSubs = subscriptionWithItems.FirstOrDefault().Items

			' Capture all the Item Ids in the subscription in the field.
			itemsInSubscription = New List(Of Tuple(Of String, Integer))()

			For Each itemInSubs In itemsInSubs
				itemsInSubscription.Add(Tuple.Create(itemInSubs.ItemName, itemInSubs.ItemId))
			Next itemInSubs
		End Sub

		''' <summary>
		''' When the window is closing, stop our time and clean up.
		''' </summary>
		''' <param name="sender"></param>
		''' <param name="e"></param>
        Private Sub Window_Closing(sender As Object, e As CancelEventArgs)
            If timer IsNot Nothing Then
                timer.IsEnabled = False
            End If

            itemSamplesFromSubscription = New List(Of ItemSampleData)()
        End Sub
#End Region
	End Class
End Namespace
