﻿Imports System.Text
Imports System.Threading.Tasks

Namespace ODataService.Simulation
	Public Class DataSample
		#Region "Private static fields"
		''' <summary>
		''' Auto-generate an always-increasing sample Id.
		''' </summary>
		Private Shared nextId As Integer = 0
		#End Region

		#Region "Private fields"
		''' <summary>
		''' The sample id.
		''' </summary>
		Private sampleId As Integer

		''' <summary>
		''' The item id that this currently samples.
		''' </summary>
'INSTANT VB NOTE: The variable itemId was renamed since Visual Basic does not allow class members with the same name:
		Private itemId_Renamed As Integer

		''' <summary>
		''' The current Value of this sample.
		''' </summary>
'INSTANT VB NOTE: The variable value was renamed since Visual Basic does not allow class members with the same name:
		Private value_Renamed As Integer

		''' <summary>
		''' The current Timestamp of this sample.
		''' </summary>
		Private timeStamp As Date

		''' <summary>
		''' The current Quality of this sample.
		''' </summary>
'INSTANT VB NOTE: The variable quality was renamed since Visual Basic does not allow class members with the same name:
		Private quality_Renamed As DataQuality
		#End Region

		#Region "Public properties"
		''' <summary>
		''' Gets the Item Id of this sample.
		''' </summary>
		Public ReadOnly Property Id() As Integer
			Get
				Return sampleId
			End Get
		End Property

		''' <summary>
		''' Gets or sets the Item Id of the item this is a sample of.
		''' </summary>
		Public Property ItemId() As Integer
			Get
				Return itemId_Renamed
			End Get
			Set(ByVal value As Integer)
				Me.itemId_Renamed = value
			End Set
		End Property

		''' <summary>
		''' Gets or sets the current Value of this sample.
		''' </summary>
		Public Property Value() As Integer
			Get
				Return value_Renamed
			End Get
			Set(ByVal value As Integer)
				Me.value_Renamed = value
			End Set
		End Property

		''' <summary>
		''' Gets or sets the current Timestamp of this sample.
		''' </summary>
		Public Property Time() As Date
			Get
				Return timeStamp
			End Get
			Set(ByVal value As Date)
				timeStamp = value
			End Set
		End Property

		''' <summary>
		''' Gets or sets the current Quality of this sample.
		''' </summary>
		Public Property Quality() As DataQuality
			Get
				Return quality_Renamed
			End Get
			Set(ByVal value As DataQuality)
				quality_Renamed = value
			End Set
		End Property
		#End Region

		#Region "Constructors"
		''' <summary>
		''' Initializes a new instance of the <see cref="DataSample"/> class
		''' Initializing constructor, provides values for the read-only properties.
		''' </summary>
		''' <param name="id">The permanent Id of this sample</param>
		Public Sub New(ByVal id As Integer)
			Me.sampleId = id

			itemId_Renamed = 0
			value_Renamed = 0
			timeStamp = Date.Now
			quality_Renamed = DataQuality.QaulityUncertain
		End Sub

		''' <summary>
		''' Initializes a new instance of the <see cref="DataSample"/> class.
		''' Auto-generate the next available sample Id, and default the other fields.
		''' </summary>
		Public Sub New(ByVal item As DataItem)
			sampleId = nextId
			nextId += 1

			itemId_Renamed = item.Id
			value_Renamed = item.Value
			timeStamp = item.Time
			quality_Renamed = item.Quality
		End Sub
		#End Region
	End Class
End Namespace
