﻿Namespace ODataService.Simulation
	''' <summary>
	''' A static container of all items in the backing data provider.
	''' </summary>
	Public NotInheritable Class ItemRegistry
		#Region "Private static fields"
		''' <summary>
		''' The internal container of items, stored by id.
		''' </summary>
'INSTANT VB NOTE: The variable items was renamed since Visual Basic does not allow class members with the same name:
		Private Shared items_Renamed As New Dictionary(Of Integer, DataItem)()

		''' <summary>
		''' The actual simulation with raw data and algorithms.
		''' </summary>
		Private Shared simulationSource As New DataSource(200)
		#End Region

		#Region "Public static properties"
		''' <summary>
		''' Public accessor for the internal items.
		''' </summary>
		Private Sub New()
		End Sub
		Public Shared ReadOnly Property Items() As IList(Of DataItem)
			Get
				Return items_Renamed.Values.ToList()
			End Get
		End Property
		#End Region

		#Region "Public static methods"
		''' <summary>
		''' Make an item with the specified Id.  Even if it already
		''' exists, force its name to the specified name.
		''' </summary>
		''' <param name="id">The Id of the item to make</param>
		''' <param name="name">The name to associate with the item Id.  If the item Id exists, change its name to this.</param>
		''' <returns>The DataItem made and registered, or found, or null if the item Id was out of range</returns>
		Public Shared Function MakeItem(ByVal id As Integer, ByVal name As String) As DataItem
			If id < DataSource.MaxItemIndex Then
				' Attempt to find an existing item from the provided Id.
				Dim item As DataItem
				If Not items_Renamed.TryGetValue(id, item) Then
					' We could not find the requested item, make it and add it.
					item = New DataItem(id, name, simulationSource)
					items_Renamed.Add(id, item)
				Else
					' The item Id already existed, change its name.
					item.Name = name
				End If

				' Return the item we made or found.
				Return item
			End If

			' The item Id was out of range.
			Return Nothing
		End Function

		''' <summary>
		''' Get or make an items.  If the item already existed in the registry, 
		''' get a reference.  If not, make a new one and add it.
		''' </summary>
		''' <param name="id">The item Id to get an item for</param>
		''' <returns>The requested item, or null if the item Id does not exist</returns>
		Public Shared Function GetItem(ByVal id As Integer) As DataItem
			' A return value.
			Dim item As DataItem = Nothing

			If id < DataSource.MaxItemIndex Then
				' Attempt to find an existing item from the provided Id.
				items_Renamed.TryGetValue(id, item)
			End If

			' Return whatever we got, including null.
			Return item
		End Function
		#End Region
	End Class
End Namespace
