﻿Namespace ODataService.Simulation
	''' <summary>
	''' A static container of all subscriptions in the backing data provider.
	''' </summary>
	Public NotInheritable Class SubscriptionRegistry
		#Region "Private static fields"
		''' <summary>
		''' The internal container of subscriptions, stored by id.
		''' </summary>
'INSTANT VB NOTE: The variable subscriptions was renamed since Visual Basic does not allow class members with the same name:
		Private Shared subscriptions_Renamed As New Dictionary(Of Integer, DataSubscription)()

		#End Region

		#Region "Public static properties"
		''' <summary>
		''' Public accessor for the internal subscriptions.
		''' </summary>
		Private Sub New()
		End Sub
		Public Shared ReadOnly Property Subscriptions() As IList(Of DataSubscription)
			Get
				Return subscriptions_Renamed.Values.ToList()
			End Get
		End Property
		#End Region

		#Region "Public static methods"
		''' <summary>
		''' Get or make a subscription.  If the subscription already existed
		''' in the registry, get a reference.  If not, make a new one and add it.
		''' </summary>
		''' <param name="id">The subscription Id to get a subscription for</param>
		''' <returns>Alway return a subscription, never fails</returns>
		Public Shared Function GetSubscription(ByVal id As Integer) As DataSubscription
			' Attempt to find an existing subscription from the provided Id.
			Dim subscription As DataSubscription
			If Not subscriptions_Renamed.TryGetValue(id, subscription) Then
				' We could not find the requested subscription, make it and add it.
				subscription = New DataSubscription(id, 1000)
				subscriptions_Renamed.Add(id, subscription)
			End If

			' Return the subscription we made or found.
			Return subscription
		End Function

		''' <summary>
		''' Delete the specified subscription, if it exists.
		''' </summary>
		''' <param name="id">The subscription Id to delete</param>
		''' <returns>True if the subscription existed and was deleted, false if it did not exist</returns>
		Public Shared Function DeleteSubscription(ByVal id As Integer) As Boolean
			' Attempt to find an existing subscription from the provided Id.
			Dim subscription As DataSubscription
			If subscriptions_Renamed.TryGetValue(id, subscription) Then
				' Turn the timer off.
				subscription.UpdateInterval = 0

				' Remove references to the subscription.
				subscriptions_Renamed.Remove(id)
			End If

			' The requested subscription did not exist.
			Return False
		End Function
		#End Region

	End Class
End Namespace
